<?php
include_once ( WPT_PLUGIN_DIR."/includes/libs/phpquery.php" );

function wpt_amazon_options_admin($self) {
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_amazon_post_optional_settings($self, $form_data=array(), $ct = '') {
  include(dirname(__file__)."/wptm-options.php");
  include(dirname(__file__)."/templates/post-optional-settings.php");
}

function wpt_amazon_campaign_type($self, $campaign_data) {
  global $wpt;
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/campaign-type.php");
}

function wpt_amazon_tag_replacements($content) {
  return $content;
}

function wpt_amazon_aws_response($params) {
  # Ulrich Mierendorff's AWS Request Method
  # http://mierendo.com/software/aws_signed_query/
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $request_verb = "GET";
  $aws_host = "ecs.amazonaws.".$options['wpt_amazon_site'];
  $uri = "/onca/xml";

  $default_params = array(
    "Service" => "AWSECommerceService",
    "AssociateTag" => $options['wpt_amazon_affkey'],
    "AWSAccessKeyId" => $options['wpt_amazon_apikey'],
    "Timestamp" => gmdate("Y-m-d\TH:i:s\Z"),
    "Version" => "2009-03-31"
  );

  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $param = str_replace("%7E", "~", rawurlencode($param));
    $value = str_replace("%7E", "~", rawurlencode($value));
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  
  $string_to_sign = $request_verb."\n".$aws_host."\n".$uri."\n".$request_query;
  $signature = base64_encode(hash_hmac("sha256", $string_to_sign, $options['wpt_amazon_secretkey'], True));
  $signature = str_replace("%7E", "~", rawurlencode($signature));
  $request = "http://".$aws_host.$uri."?".$request_query."&Signature=".$signature;

  return wpt_get_xml_response( $request, "Amazon" );
}

function wpt_amazon_get_user_rating( $url ) {
  $response = wpt_get_response( $url, "Amazon" );
  $_amazon_user_rating = phpQuery::newDocument( $response );
  $avg_rating = $_amazon_user_rating->find(".crAvgStars .asinReviewsSummary span.swSprite")->attr("title");
  return preg_replace("/^(\d\.\d).*/", "$1", $avg_rating);
}

function wpt_amazon_star_rating($rating) {
  $rating_percentage = ($rating/5)*100;
  $star_rating = '<span class="wpt-amazon-rating"><span class="wpt-amazon-rating-gold" style="width: '.$rating_percentage.'%;"></span></span>';
  return $star_rating;
}

function wpt_amazon_post($params) {
  global $wpdb, $wpt;
  extract($params);
  
  include(dirname(__file__)."/wptm-options.php");
  $amazon_department = $campaign_meta["amazon_department"];
  if($amazon_department == "")
    $amazon_department = "All";
  
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_amazon_template_tags );
  }
  
  if($keywords == "" && $browsenode  == "") {
    return send_module_errors("Amazon", "[Keywords Empty] No keyword and/or BrowseNode specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'amazon'");
  if($template == false || empty($template)) {
    return send_module_errors("Amazon", "[Keywords Empty] Module Template was not found.");
  }

  $output = array();

  if($browsenode == "") $browsenode = 0;

  for($i = 0; $i < $count; $i++) {
    $response_params_array = array(
                                    "Operation" => "ItemSearch",
                                    "Keywords" => urlencode($keywords),
                                    "SearchIndex" => "All",
                                    "MerchantId" => "All",
                                    "ItemPage" => $page,
                                    "ResponseGroup" => "Large"
                                  );
    if( $amazon_department != "All" ) 
      $response_params_array['SearchIndex'] = $amazon_department;
    if( $browsenode != 0 )
      $response_params_array['BrowseNode'] = $browsenode;

    $aws_xml_response = wpt_amazon_aws_response( $response_params_array );
    
    if(!empty($aws_xml_response["error"]))
      return $aws_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter1 = 1;
    $counter2 = 0;

    if (!$aws_xml_response) {
      return send_module_errors("Amazon", "[Request Failed] API request failed." );
    }

    if (!empty($aws_xml_response->Error)) {
      return send_module_errors("Amazon", "[API Failed] Amazon API request failed. API Error: ({$aws_xml_response->Error->Code}), {$aws_xml_response->Error->Message}");
    }

    if (!empty($aws_xml_response->Items->Request->Errors->Error->Code)) {
      return send_module_errors("Amazon", "[API Failed] Amazon API request failed. Amazon Error: ({$aws_xml_response->Items->Request->Errors->Error->Code}), {$aws_xml_response->Items->Request->Errors->Error->Message}");
    }

    if (!$aws_xml_response->Items->Item || count( $aws_xml_response->Items->Item ) == 0) {
      return send_module_errors("Amazon", "[Content Empty] No products found for this keyword.");
    }
    
    foreach($aws_xml_response->Items->Item as $item) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      $is_image_less_post = false;
      
      if( $wpt->options["wpt_amazon_exclude_imageless_products"] == "Yes"
        && ( empty( $item->MediumImage->URL ) || empty( $item->SmallImage->URL )) ) {
        $is_image_less_post = true;
      }
      

      if ($counter2<$count_this && !$is_image_less_post) {
        $description = "";
        if (!empty($item->EditorialReviews->EditorialReview)) {
          foreach($item->EditorialReviews->EditorialReview as $editorial_review) {
            $description.= $editorial_review->Content;
          }
        }

        $features = "";
        if (!empty($item->ItemAttributes->Feature)) {
          $features_array = array();
          foreach($item->ItemAttributes->Feature as $feature) {
            $features_array[] = "<li>$feature</li>";
          }
          $features = '<ul>'.implode("\n", $features_array).'</ul>';
        }

        $thumbnail = '<a href="'.$item->DetailPageURL.'"><img class="wptm-amazon-alignleft alignleft" src="'.(empty($item->MediumImage->URL) ? $item->SmallImage->URL : $item->MediumImage->URL).'" /></a>';
        $thumbnail_url = (empty($item->MediumImage->URL) ? $item->SmallImage->URL : $item->MediumImage->URL);

        $price = str_replace("$", "$ ", $item->OfferSummary->LowestNewPrice->FormattedPrice);
        $listprice = str_replace("$", "$ ", $item->ItemAttributes->ListPrice->FormattedPrice);
        
        if(empty($price)) 
          $price = str_replace("$", "$ ", $item->ItemAttributes->ListPrice->LowestRefurbishedPrice);
        
        if(empty($listprice))
          $listprice = str_replace("$", "$ ", $item->ItemAttributes->ListPrice->LowestRefurbishedPrice);
        
        if(empty($price)) 
          $price = str_replace("$", "$ ", $item->ItemAttributes->ListPrice->LowestUsedPrice);
        
        if(empty($listprice))
          $listprice = str_replace("$", "$ ", $item->ItemAttributes->ListPrice->LowestUsedPrice);
        
        $content = $template;

        $content_search_tags = $wpt_amazon_template_tags;
        
        $content_replace_tags = array(
                                      $keywords,
                                      (string)$item->ItemAttributes->Title,
                                      $description,
                                      $features,
                                      $thumbnail,
                                      '<img src="'.(string)$item->SmallImage->URL.'" alt="" />',
                                      '<img src="'.(string)$item->MediumImage->URL.'" alt="" />',
                                      '<img src="'.(string)$item->LargeImage->URL.'" alt="" />',
                                      $thumbnail_url,
                                      (string)$item->SmallImage->URL,
                                      (string)$item->MediumImage->URL,
                                      (string)$item->LargeImage->URL,
                                      '<a href="'.(string)$item->DetailPageURL.'"><img src="'.$wpt->options['wpt_amazon_buy_now_image'].'" alt="Buy Now!" style="border: none;" /></a>',
                                      $price,
                                      $listprice,
                                      (string)$item->DetailPageURL,
                                      wpt_amazon_star_rating(wpt_amazon_get_user_rating((string)$item->ItemLinks->ItemLink[5]->URL)),
                                      '<a href="'.$item->DetailPageURL.'">'.$item->ItemAttributes->Title.'</a>',
                                      (string)$item->ASIN
                                    );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5($item->ASIN),
          "title" => $item->ItemAttributes->Title,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_amazon_wp_head() {
  wp_enqueue_style("wptm-amazon-css", WPT_PLUGIN_URL."modules/wptm-amazon/css/wptm-amazon.css");
}

function wpt_amazon_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_amazon_module_template_installed") != "Yes") {
    $wpt->set_template("amazon", $wpt_amazon_default_module_template);
    update_option("wpt_amazon_module_template_installed", "Yes");
  }
  if(get_option("wpt_amazon_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_amazon_default_post_template);
    update_option("wpt_amazon_post_template_installed", "Yes");
  }
}

function wpt_amazon_get_browsenode_name($browsenode) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  $aws_xml_response = wpt_amazon_aws_response(array(
                          "Operation" => "BrowseNodeLookup",
                          "BrowseNodeId" => $browsenode,
                          "ResponseGroup" => "BrowseNodeInfo"
                      ));
  if ($aws_xml_response === false) {
    return false;
  } else {
    if($aws_xml_response->BrowseNodes->BrowseNode->Name) {
      return $aws_xml_response->BrowseNodes->BrowseNode->Name;
    } else {
      return false;
    }
  }
}

?>
