<?php

function wpt_coupons_options_admin($self) {
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_coupons_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);

  $default_params = array(
    "UserName" => $options['wpt_coupons_username'],
    "SubscriptionID" => $options['wpt_coupons_subscription_id'],
    "PageSize" => "10",
    "Sort" => "StartDateHigh",
    "RequestType" => "Codes",
    "Action" => "Search",
  );

  $params = array_merge($params, $default_params);
  ksort($params);
  
  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  
  $request = "http://webservices.icodes.co.uk/ws2.php?".$request_query;

  return wpt_get_xml_response( $request, "Coupons" );
}

function wpt_coupons_post($params) {
  global $wpdb, $wpt;
  
  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_coupons_template_tags );
  }
  
  if($keywords == "") {
    return send_module_errors("Coupons", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'coupons'");
  if($template == false || empty($template)) {
    return send_module_errors("Coupons", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $coupons_xml_response = wpt_coupons_response(array(
                                        "Query" => urlencode($keywords),
                                        "Page" => $page,
                                      ));
    if(!empty($coupons_xml_response["error"]))
      return $coupons_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if ( $coupons_xml_response->Results < 1 && $coupons_xml_response->Message == "No Results") {
      return send_module_errors("Coupons", "[Content Empty] No coupons found for this keyword.");
    }

    if ($coupons_xml_response->Results < 1) {
      return send_module_errors("Coupons", "[API Failed] API request failed. API Message: {$coupons_xml_response->Message}" );
    }
    
    
    
    foreach($coupons_xml_response->item as $coupon) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;

      if ($counter2<$count_this) {
        $content = $template;
        $content_search_tags = $wpt_coupons_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$coupon->icid,
          (string)$coupon->mid,
          (string)$coupon->title,
          (string)$coupon->description,
          (string)$coupon->merchant,
          (string)$coupon->merchant_logo_url,
          (string)$coupon->network,
          (string)$coupon->voucher_code,
          (string)$coupon->excode,
          (string)$coupon->start_date,
          (string)$coupon->expiry_date,
          (string)$coupon->affiliate_url,
          (string)$coupon->merchant_url,
          (string)$coupon->category,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5((string)$coupon->icid),
          "title" => (string)$coupon->title,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );

        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_coupons_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_coupons_module_template_installed") != "Yes") {
    $wpt->set_template("coupons", $wpt_coupons_default_module_template);
    update_option("wpt_coupons_module_template_installed", "Yes");
  }
  if(get_option("wpt_coupons_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_coupons_default_post_template);
    update_option("wpt_coupons_post_template_installed", "Yes");
  }
}
?>
