<?php

function wpt_eventful_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_eventful_post_optional_settings($self, $form_data=array(), $ct = '') {
  include(dirname(__file__)."/wptm-options.php");
  include(dirname(__file__)."/templates/post-optional-settings.php");
}

function wpt_eventful_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  $default_params = array(
    "app_key" => $options['wpt_eventful_apikey'],
    "page_size" => "10",
  );
  
  if( isset( $params['eventful_date'] ) ) {
    $default_params['date'] = $params['eventful_date'];
    unset( $params['eventful_date'] );
  }
  
  if( isset( $params['eventful_category'] ) ) {
    $default_params['category'] = $params['eventful_category'];
    unset( $params['eventful_category'] );
  }
  
  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);

  $request = "http://api.eventful.com/rest/events/search?".$request_query;

  return wpt_get_xml_response( $request, "Eventful" );
}

function wpt_eventful_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_eventful_template_tags );
  }
  
  if($keywords == "") {
    return send_module_errors("Eventful", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'eventful'");
  if($template == false || empty($template)) {
    return send_module_errors("Eventful", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $response_array = array(
                            "keywords" => urlencode($keywords),
                            "page_number" => $page,
                          );

    if( isset( $campaign_meta['eventful_category'] ) && !empty( $campaign_meta['eventful_category'] ) )
      $response_array['eventful_category'] = $campaign_meta['eventful_category'];

    $eventful_xml_response = wpt_eventful_response($response_array);

    if(!empty($eventful_xml_response["error"]))
      return $eventful_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$eventful_xml_response) {
      return send_module_errors("Eventful", "[Request Failed] API request failed." );
    }

    if (!isset($eventful_xml_response->events) || empty( $eventful_xml_response->events ) || count($eventful_xml_response->events->event) < 1) {
      return send_module_errors("Eventful", "[Request Failed] {$eventful_xml_response->description}." );
    }


    foreach($eventful_xml_response->events->event as $event) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;

      if ($counter2<$count_this) {
        $content = $template;
        
        $content_search_tags = $wpt_eventful_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$event->title,
          (string)$event->url,
          (string)$event->description,
          (string)$event->start_time,
          (string)$event->stop_time,
          (string)$event->venue_id,
          (string)$event->venue_url,
          (string)$event->venue_name,
          (string)$event->venue_address,
          (string)$event->city_name,
          (string)$event->region_name,
          (string)$event->region_abbr,
          (string)$event->postal_code,
          (string)$event->country_name,
          (string)$event->country_abbr2,
          (string)$event->country_abbr,
          (string)$event->latitude,
          (string)$event->longitude,
          (string)$event->owner,
          (string)$event->image->url,
          (string)$event->image->thumb->url,
          (string)$event->image->small->url,
          (string)$event->image->medium->url,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5($event['id']),
          "title" => $event->title,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_eventful_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_eventful_module_template_installed") != "Yes") {
    $wpt->set_template("eventful", $wpt_eventful_default_module_template);
    update_option("wpt_eventful_module_template_installed", "Yes");
  }
  if(get_option("wpt_eventful_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_eventful_default_post_template);
    update_option("wpt_eventful_post_template_installed", "Yes");
  }
}
?>
