<?php

function wpt_flickr_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_flickr_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $default_params = array(
    "method"=>"flickr.photos.search",
    "license"=>$options['wpt_flickr_licenses'],
    "api_key"=>$options['wpt_flickr_apikey'],
    "per_page"=>"10",
    "sort"=> "relevance",
    "extras"=>urlencode('date_taken,owner_name,tags,url_sq,url_t,url_s,url_m,url_o,description')
  );

  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  $request = "http://api.flickr.com/services/rest/?".$request_query;

  return wpt_get_xml_response( $request, "Flickr" );
}

function wpt_flickr_post($params) {
  global $wpdb, $wpt;
  
  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_flickr_template_tags );
  }
  
  if($keywords == "") {
    return send_module_errors("Flickr", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'flickr'");
  if($template == false || empty($template)) {
    return send_module_errors("Flickr", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $flickr_xml_response = wpt_flickr_response(array(
                                      "text" => urlencode($keywords),
                                      "page" => $page,
                                      ));
    if(!empty($flickr_xml_response["error"]))
      return $flickr_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$flickr_xml_response) {
      return send_module_errors("Flickr", "[Request Failed] API request failed." );
    }

    if (isset($flickr_xml_response->err)) {
      $message = '<p>Error in Flickr API request. Flickr Error:</p>
      <p><strong>'.$flickr_xml_response->err['code'].':</strong> '.$flickr_xml_response->err['msg'].'</p>';
      return send_module_errors("Flickr", "[API Failed] {$message}");
    }
    
    if (!$flickr_xml_response->photos->photo) {
      return send_module_errors("Flickr", "[Content Empty] No photos found for this keyword.");
    }
    
    foreach($flickr_xml_response->photos->photo as $photo) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      if ($counter2<$count_this) {
        $content = $template;

        $content_search_tags = $wpt_flickr_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$photo['title'],
          (string)$photo->description,
          '<img src="'.(string)$photo['url_o'].'" alt="'.(string)$photo['title'].' Original" />',
          '<img src="'.(string)$photo['url_t'].'" alt="'.(string)$photo['title'].' Thumbnail" />',
          '<img src="'.(string)$photo['url_m'].'" alt="'.(string)$photo['title'].' Medium" />',
          '<img src="'.(string)$photo['url_s'].'" alt="'.(string)$photo['title'].' Small" />',
          '<img src="'.(string)$photo['url_sq'].'" alt="'.(string)$photo['title'].' Square" />',
          (string)$photo['url_o'],
          (string)$photo['url_t'],
          (string)$photo['url_m'],
          (string)$photo['url_s'],
          (string)$photo['url_sq'],
          (string)$photo['ownername'],
          (string)$photo['datetaken'],
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);


        $post_data = array(
          "uid" => md5((string)$photo['url_o'].(string)$photo['title']),
          "title" => (string)$photo['title'],
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_flickr_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_flickr_module_template_installed") != "Yes") {
    $wpt->set_template("flickr", $wpt_flickr_default_module_template);
    update_option("wpt_flickr_module_template_installed", "Yes");
  }
  if(get_option("wpt_flickr_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_flickr_default_post_template);
    update_option("wpt_flickr_post_template_installed", "Yes");
  }
}
?>
