<?php

function wpt_heyzap_options_admin($self) {
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_heyzap_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);

  $default_params = array(
    "site_key" => $options['wpt_heyzap_site_key'],
    "secret_key" => $options['wpt_heyzap_secret_key'],
    "per_page" => "10",
    "swf_only" => "false",
    "format" => "xml"
  );

  $params = array_merge($params, $default_params);
  ksort($params);
  
  $available_categories = array("featured", "hot_new", "popular", "sports", "shooter", "strategy", "puzzle", "action", "multiplayer", "adventure", "lifestyle", "social", "new");
  
  $find_games_by_category = false;
  $kw = str_replace( " ", "_", trim(strtolower($params["keyword"])) );
  if( in_array($kw, $available_categories) ) {
    $find_games_by_category = true;
    $params["category"] = trim(strtolower($params["keyword"]));
    unset($params["keyword"]);
  }

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  
  if( $find_games_by_category )
    $request = "http://www.heyzap.com/publisher_api/v2/games_by_category?".$request_query;
  else
    $request = "http://www.heyzap.com/publisher_api/v2/games_by_keyword?".$request_query;

  return wpt_get_xml_response( $request, "Heyzap" );
}

function wpt_heyzap_star_rating($rating) {
  $rating_percentage = ($rating/5)*100;
  $star_rating = '<span class="wpt-heyzap-rating"><span class="wpt-heyzap-rating-gold" style="width: '.$rating_percentage.'%;"></span></span>';
  return $star_rating;
}

function wpt_heyzap_post($params) {
  global $wpdb, $wpt;
  
  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_heyzap_template_tags );
  }
  
  if($keywords == "") {
    return send_module_errors("Heyzap", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'heyzap'");
  if($template == false || empty($template)) {
    return send_module_errors("Heyzap", "[Template Missing] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $hzp_xml_response = wpt_heyzap_response(array(
                                        "keyword" => urlencode($keywords),
                                        "page" => $page,
                                      ));
    if(!empty($hzp_xml_response["error"]))
      return $hzp_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$hzp_xml_response) {
      return send_module_errors("Heyzap", "[Request Failed] API request failed." );
    }

    if ( (count($hzp_xml_response->games->game) < 1) || !$hzp_xml_response->games->game) {
      return send_module_errors("Heyzap", "[Content Empty] No games found for this keyword.");
    }
    
    foreach($hzp_xml_response->games->game as $game) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;

      if ($counter2<$count_this) {
        $content = $template;
        $description = "{$game->description}";
        $description = str_replace( array("\\t", "\\n"), array("\t","\n"), $description );
        $description = nl2br( $description );
        
        $content_search_tags = $wpt_heyzap_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$game->display_name,
          (string)$game->embed_code,
          (string)$game->categories,
          '<img src="'.(string)$game->thumb_100x100.'" alt="'.(string)$game->display_name.'" />',
          (string)$game->thumb_100x100,
          (string)$game->width,
          (string)$game->last_updated,
          (string)$description,
          '<img src="'.(string)$game->thumb_640x480.'" alt="'.(string)$game->display_name.'" />',
          (string)$game->thumb_640x480,
          wpt_heyzap_star_rating((string)$game->rating),
          (string)$game->name
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5($game->name),
          "title" => $game->display_name,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );

        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_heyzap_wp_head() {
  wp_enqueue_style("wptm-heyzap-css", WPT_PLUGIN_URL."modules/wptm-heyzap/css/wptm-heyzap.css");
}

function wpt_heyzap_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_heyzap_module_template_installed") != "Yes") {
    $wpt->set_template("heyzap", $wpt_heyzap_default_module_template);
    update_option("wpt_heyzap_module_template_installed", "Yes");
  }
  if(get_option("wpt_heyzap_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_heyzap_default_post_template);
    update_option("wpt_heyzap_post_template_installed", "Yes");
  }
}
?>
