<?php
include_once ( WPT_PLUGIN_DIR."/includes/libs/phpquery.php" );

function wpt_imdb_options_admin($self) {
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_imdb_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  libxml_use_internal_errors(true);
  $default_params = array(
    "count" => 10,
  );

  $params = array_merge($params, $default_params);
  ksort($params);

  $available_genres = array("action", "adventure", "animation", "biography", "comedy", "crime", "documentary", "drama", "family", "fantasy", "film_noir", "game_show", "history", "horror", "music", "musical", "mystery", "news", "reality_tv", "romance", "sci_fi", "sport", "talk_show", "thriller", "war", "western");

  $kw = str_replace( " ", "_", trim(strtolower(urldecode($params["title"]))) );
  if( in_array($kw, $available_genres) ) {
    $params["genres"] = urlencode($kw);
    $params["title_type"] = 'feature';
    unset($params["title"]);
  }

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  
  $request = "http://www.imdb.com/search/title?".$request_query;

  $response = wpt_get_response( $request, "IMDB" );
  $_response = phpQuery::newDocument($response);
  $output = array();
  $_imdb_rows = $_response->find("table.results")->find("tr.detailed");
  foreach( $_imdb_rows as $imdb_row ) {
    $_imdb_row = pq($imdb_row);
    $credits = str_replace(array("\r", "\n"), array("", ""), $_imdb_row->find("td.title span.credit")->text());
    preg_match( "/Dir:(.*)With:(.*)/", $credits, $credit_matches );
    $genre = explode("|", $_imdb_row->find("td.title span.genre")->text());
    $gen = array();
    foreach( $genre as $g ) {
      $gen[] = trim($g);
    }
    $episode_exists = count($_imdb_row->find("td.title span.episode"));
    if($episode_exists > 0) {
      $imdb_url = $_imdb_row->find("td.title span.episode>a[href^='/title']")->attr("href");
    } else {
      $imdb_url = $_imdb_row->find("td.title>a[href^='/title']")->attr("href");
    }
    $imdb_url = wpt_imdb_absolutize_url("http://www.imdb.com", $imdb_url);
    $year_type = str_replace(array("\r", "\n"), array("", ""), $_imdb_row->find("td.title span.year_type")->text());
    $year_type = preg_replace("/\(([^\)]+)\)/", "$1", $year_type);
    $year = preg_replace("/^.*(\d{4}).*$/", "$1", $year_type);
    $type = str_replace($year, "", $year_type);
    $rating_data = explode("|", $_imdb_row->find("td.title div.user_rating>div.rating-list")->attr("id"));
    $rating = $rating_data[2];
    $output['movie'][] = (object)array(
      "thumbnail_url" => wpt_imdb_absolutize_url("http://ia.media-imdb.com", $_imdb_row->find("td.image img")->attr("src")),
      "url" => $imdb_url,
      "title" => $_imdb_row->find("td.title>a[href^='/title']")->text(),
      "episode" => $_imdb_row->find("td.title span.episode")->text(),
      "year" => $year,
      "type" => $type,
      "outline" => $_imdb_row->find("td.title span.outline")->text(),
      "director" => trim($credit_matches[1]),
      "cast" => trim($credit_matches[2]),
      "genre" => join(", ", $gen),
      "run_time" => $_imdb_row->find("td.title span.runtime")->text(),
      "certificate_image_url" => $_imdb_row->find("td.title span.certificate img")->attr("src"),
      "rating" => $rating,
    );
  }
  return (object)$output;
}

function wpt_imdb_absolutize_url($base, $url) {
  if( preg_match("/\/realm\//",$url) ) {
    return "http://www.imdb.com".$url;
  }
  if( strstr( $url, $base ) )
    return $url;
  else {
    $new_url = $base."/".preg_replace("/^\//", "", $url);
    return $new_url;
  }
}

function wpt_imdb_star_rating($rating) {
  $rating_percentage = ($rating/10)*100;
  $star_rating = '<span class="wpt-imdb-rating"><span class="wpt-imdb-rating-gold" style="width: '.$rating_percentage.'%;"></span></span>'.' <span>'.$rating.'/10</span>';
  return $star_rating;
}

function wpt_imdb_post($params) {
  global $wpdb, $wpt;
  
  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_imdb_template_tags );
  }
  
  if($keywords == "") {
    return send_module_errors("IMDB", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );
  $next_start = (($page - 1) * 10) + 1;

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'imdb'");
  if($template == false || empty($template)) {
    return send_module_errors("IMDB", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $imdb_response = wpt_imdb_response(array(
                                        "title" => strtolower(urlencode($keywords)),
                                        "start" => $next_start,
                                      ));
    $count_this = ($count <= 25) ? $count : 25;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$imdb_response) {
      return send_module_errors("IMDB", "[Request Failed] API request failed." );
    }

    if ( (count($imdb_response->movie) < 1)) {
      return send_module_errors("IMDB", "[Content Empty] No Movies found for this keyword.");
    }
    
    foreach($imdb_response->movie as $movie) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;

      if ($counter2<$count_this) {
        $content = $template;
        
        $content_search_tags = $wpt_imdb_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          $movie->thumbnail_url,
          $movie->url,
          $movie->title,
          $movie->episode,
          $movie->year,
          $movie->type,
          $movie->outline,
          $movie->director,
          $movie->cast,
          $movie->genre,
          $movie->run_time,
          wpt_imdb_star_rating($movie->rating),
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5($movie->url),
          "title" => $movie->title,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_imdb_wp_head() {
  wp_enqueue_style("wptm-imdb-css", WPT_PLUGIN_URL."modules/wptm-imdb/css/wptm-imdb.css");
}

function wpt_imdb_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_imdb_module_template_installed") != "Yes") {
    $wpt->set_template("imdb", $wpt_imdb_default_module_template);
    update_option("wpt_imdb_module_template_installed", "Yes");
  }
  if(get_option("wpt_imdb_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_imdb_default_post_template);
    update_option("wpt_imdb_post_template_installed", "Yes");
  }
}
?>
