<?php

function wpt_itunes_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_itunes_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $default_params = array(
    "country"=>$options['wpt_itunes_country'],
    "resultEntity"=>$options['wpt_itunes_entity'],
  );

  $params = array_merge($params, $default_params);
  if( $options['wpt_itunes_entity'] == "all" || empty( $options['wpt_itunes_entity'] ) ) {
    unset( $params['resultEntity'] );
  }
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  $request = "http://ax.itunes.apple.com/WebObjects/MZStoreServices.woa/wa/wsSearch?".$request_query;

  $json_response = wpt_get_response( $request, "Itunes" );

  return json_decode( $json_response );
}

function wpt_itunes_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_itunes_template_tags );
  }

  if($keywords == "") {
    return send_module_errors("Itunes", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'itunes'");
  if($template == false || empty($template)) {
    return send_module_errors("Itunes", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();
  
  for($i = 0; $i < $count; $i++) {
    $itunes_json_response = wpt_itunes_response(array(
                                      "term" => urlencode($keywords),
                                      "limit" => $last_post_number + 10,
                                      ));
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$itunes_json_response) {
      return send_module_errors("Itunes", "[Request Failed] API request failed." );
    }

    if(empty($itunes_json_response->results) || count( $itunes_json_response->results ) < 1)
      return send_module_errors("Itunes", "[Empty Content] No items were found for the keyword.");
    
    if( isset( $itunes_json_response->errorMessage ) && !empty( $itunes_json_response->errorMessage ) ) 
      return send_module_errors("Itunes", "[Request Failed] The following API error occurred: {$itunes_json_response->errorMessage}.");
    
    foreach($itunes_json_response->results as $itunes_item) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      $screenshots_gallery = array();
      if( isset($itunes_item->screenshotUrls) && count($itunes_item->screenshotUrls) > 0 ) {
        foreach( $itunes_item->screenshotUrls as $screenshotUrl ) {
          $screenshots_gallery[] = '<span class="itunes-screenshots"><img src="'.$screenshotUrl.'" alt="" /></span>';
        }
      }
      $screenshots_gallery = join("\n", $screenshots_gallery);
      $ipad_screenshots_gallery = array();
      if( isset($itunes_item->ipadScreenshotUrls) && count($itunes_item->ipadScreenshotUrls) > 0 ) {
        foreach( $itunes_item->ipadScreenshotUrls as $ipadScreenshotUrl ) {
          $ipad_screenshots_gallery[] = '<span class="itunes-ipad-screenshots"><img src="'.$ipadScreenshotUrl.'" alt="" /></span>';
        }
      }
      $ipad_screenshots_gallery = join("\n", $ipad_screenshots_gallery);
      if ($counter2<$count_this) {
        $content = $template;

        $content_search_tags = $wpt_itunes_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          $itunes_item->wrapperType,
          $itunes_item->kind,
          $itunes_item->trackName,
          $itunes_item->trackCensoredName,
          $itunes_item->artworkUrl60,
          $itunes_item->artworkUrl100,
          $itunes_item->artworkUrl512,
          $itunes_item->artistViewUrl,
          $itunes_item->trackViewUrl,
          $itunes_item->artistId,
          $itunes_item->trackId,
          $itunes_item->artistName,
          $itunes_item->sellerName,
          "$ ".$itunes_item->price,
          $itunes_item->sellerUrl,
          $itunes_item->fileSizeBytes,
          $itunes_item->releaseDate,
          $itunes_item->contentAdvisoryRating,
          $itunes_item->primaryGenreName,
          $itunes_item->primaryGenreId,
          $itunes_item->genres,
          $itunes_item->description,
          $itunes_item->trackContentRating,
          $screenshots_gallery,
          $ipad_screenshots_gallery,
        );

        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5((string)$itunes_item->trackId),
          "title" => (string)$itunes_item->trackName,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_itunes_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_itunes_module_template_installed") != "Yes") {
    $wpt->set_template("itunes", $wpt_itunes_default_module_template);
    update_option("wpt_itunes_module_template_installed", "Yes");
  }
  if(get_option("wpt_itunes_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_itunes_default_post_template);
    update_option("wpt_itunes_post_template_installed", "Yes");
  }
}
?>
