<?php

function wpt_jobs_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_jobs_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $default_params = array(
    "apikey"=>$options['wpt_jobs_apikey'],
    "count"=>"10",
  );

  if( isset($options['wpt_jobs_min_budget']) && !empty( $options['wpt_jobs_min_budget'] ) )
    $default_params['min_budget'] = $options['wpt_jobs_min_budget'];
  else
    unset( $default_params['min_budget'] );

  if( isset($options['wpt_jobs_max_budget']) && !empty( $options['wpt_jobs_max_budget'] ) )
    $default_params['max_budget'] = $options['wpt_jobs_max_budget'];
  else
    unset( $default_params['max_budget'] );

  if( isset($options['wpt_jobs_nonpublic']) && !empty( $options['wpt_jobs_nonpublic'] ) )
    $default_params['nonpublic'] = ($options['wpt_jobs_nonpublic'] == "Yes") ? 1 : 0;
  else
    unset( $default_params['nonpublic'] );

  if( isset($options['wpt_jobs_for_gold_members']) && !empty( $options['wpt_jobs_for_gold_members'] ) )
    $default_params['for_gold_members'] = ($options['wpt_jobs_for_gold_members'] == "Yes") ? 1 : 0;
  else
    unset( $default_params['for_gold_members'] );

  if( isset($options['wpt_jobs_trial']) && !empty( $options['wpt_jobs_trial'] ) )
    $default_params['trial'] = ($options['wpt_jobs_trial'] == "Yes") ? 1 : 0;
  else
    unset( $default_params['trial'] );

  if( isset($options['wpt_jobs_featured']) && !empty( $options['wpt_jobs_featured'] ) )
    $default_params['featured'] = ($options['wpt_jobs_featured'] == "Yes") ? 1 : 0;
  else
    unset( $default_params['featured'] );

  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  $request = "http://api.freelancer.com/Project/Search.xml?".$request_query;

  return wpt_get_xml_response( $request, "Jobs" );
}

function wpt_jobs_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_jobs_template_tags );
  }

  if($keywords == "") {
    return send_module_errors("Jobs", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'jobs'");
  if($template == false || empty($template)) {
    return send_module_errors("Jobs", "[Template Missing] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $jobs_xml_response = wpt_jobs_response(array(
                                      "keyword" => urlencode($keywords),
                                      "pg" => $page,
                                      ));
    if(!empty($jobs_xml_response["error"]))
      return $jobs_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$jobs_xml_response) {
      return send_module_errors("Jobs", "[Request Failed] API request failed." );
    }

    if (isset($jobs_xml_response->error)) {
      $message = '<p>Error in FreeLancer API request. FreeLancer API Error:</p>
      <p>'.$jobs_xml_response->error.'</p>';
      return send_module_errors("Jobs", "[API Failed] $message");
    }
    
    if (!$jobs_xml_response->items->item) {
      return send_module_errors("Jobs", "[Content Empty] No jobs found for this keyword.");
    }
    
    foreach($jobs_xml_response->items->item as $job) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      if ($counter2<$count_this) {
        $content = $template;

        $content_search_tags = $wpt_jobs_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$job->name,
          (string)$job->url,
          date_i18n( "d F Y, h:i A", (double)$job->start_unixtime ),
          (string)$job->start_date,
          date_i18n( "d F Y, h:i A", (double)$job->end_unixtime ),
          (string)$job->end_date,
          (string)$job->buyer->username,
          (string)$job->buyer->id,
          (string)$job->buyer->url,
          (string)$job->state,
          (string)$job->short_descr,
          (string)$job->budget->min,
          (string)$job->budget->max,
          (string)$job->bid_stats->count,
          (string)$job->bid_stats->avg,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5((string)$job->id),
          "title" => (string)$job->name,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_jobs_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_jobs_module_template_installed") != "Yes") {
    $wpt->set_template("jobs", $wpt_jobs_default_module_template);
    update_option("wpt_jobs_module_template_installed", "Yes");
  }
  if(get_option("wpt_jobs_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_jobs_default_post_template);
    update_option("wpt_jobs_post_template_installed", "Yes");
  }
}
?>
