<?php

function wpt_linkshare_options_admin($self) {
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_linkshare_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);

  $default_params = array(
    "token" => $options['wpt_linkshare_token'],
    "max" => "10",
  );

  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  
  $request = "http://feed.linksynergy.com/productsearch?".$request_query;

  return wpt_get_xml_response( $request, "LinkShare" );
}

function wpt_linkshare_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_linkshare_template_tags );
  }

  if($keywords == "") {
    return send_module_errors("LinkShare", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'linkshare'");
  if($template == false || empty($template)) {
    return send_module_errors("LinkShare", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $linkshare_xml_response = wpt_linkshare_response(array(
                                        "keyword" => urlencode(strtolower($keywords)),
                                        "pagenumber" => $page,
                                      ));

    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$linkshare_xml_response) {
      return send_module_errors("LinkShare", "[Request Failed] API request failed." );
    }

    if ($linkshare_xml_response->Errors) {
      return send_module_errors("LinkShare", "[Request Failed] API request failed with the following error: ".$linkshare_xml_response->Errors[0]->ErrorID.": ". $linkshare_xml_response->Errors[0]->ErrorText);
    }

    if ( (count($linkshare_xml_response->item) < 1) || !$linkshare_xml_response->item) {
      return send_module_errors("LinkShare", "[Content Empty] No linkshare items found for this keyword.");
    }
    
    foreach($linkshare_xml_response->item as $linkshare_item) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      if ($counter2<$count_this) {
        $content = $template;
        $price_currency = (string)$linkshare_item->price['currency'];
        $linkshare_price = ( empty($price_currency) ) ? 'USD' : $price_currency;
        $linkshare_price.= " ".(string)$linkshare_item->price;

        $content_search_tags = $wpt_linkshare_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$linkshare_item->productname,
          (string)$linkshare_item->merchantname,
          (string)$linkshare_item->mid,
          (string)$linkshare_item->description->short,
          (string)$linkshare_item->description->long,
          (string)$linkshare_item->linkurl,
          (string)$linkshare_item->imageurl,
           $linkshare_price,
          (string)$linkshare_item->createdon,
          (string)$linkshare_item->linkid,
          (string)$linkshare_item->sku,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5((string)$linkshare_item->linkurl),
          "title" => (string)$linkshare_item->productname,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );

        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_linkshare_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_linkshare_module_template_installed") != "Yes") {
    $wpt->set_template("linkshare", $wpt_linkshare_default_module_template);
    update_option("wpt_linkshare_module_template_installed", "Yes");
  }
  if(get_option("wpt_linkshare_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_linkshare_default_post_template);
    update_option("wpt_linkshare_post_template_installed", "Yes");
  }
}
?>
