<?php

function wpt_musixmatch_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_musixmatch_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $default_params = array(
    "format"=>"xml",
    "apikey"=>$options['wpt_musixmatch_apikey'],
    "page_size"=>"10",
    "f_has_lyrics"=>"1"
  );

  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  $request = "http://api.musixmatch.com/ws/1.1/track.search?".$request_query;

  return wpt_get_xml_response( $request, "Musixmatch" );
}

function wpt_musixmatch_get_lyrics($track_id) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $params = array(
    "format"=>"xml",
    "apikey"=>$options['wpt_musixmatch_apikey'],
    "track_id"=>$track_id,
  );

  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  $request = "http://api.musixmatch.com/ws/1.1/track.lyrics.get?".$request_query;

  return wpt_get_xml_response( $request, "Musixmatch" );
}

function wpt_musixmatch_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_musixmatch_template_tags );
  }

  if($keywords == "") {
    return send_module_errors("Musixmatch", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'musixmatch'");
  if($template == false || empty($template)) {
    return send_module_errors("Musixmatch", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $musixmatch_xml_response = wpt_musixmatch_response(array(
                                      "q" => urlencode($keywords),
                                      "page" => $page,
                                      ));
    if(!empty($musixmatch_xml_response["error"]))
      return $musixmatch_xml_response;
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$musixmatch_xml_response) {
      return send_module_errors("Musixmatch", "[Request Failed] API request failed." );
    }

    if ((string)$musixmatch_xml_response->header->status_code != "200") {
      $message = '<p>Error in Musixmatch API request. Musixmatch Error Code:</p>
      <p><strong>'.$musixmatch_xml_response->header->status_code.':</strong></p>';
      return send_module_errors("Musixmatch", "[API Failed] $message");
    }

    if (!$musixmatch_xml_response->body->track_list->track) {
      return send_module_errors("Musixmatch", "[Content Empty] No tracks found for this keyword.");
    }
    
    foreach($musixmatch_xml_response->body->track_list->track as $track) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      if($counter2<$count_this) {
        $content = $template;
        
        $track_lyrics_xml_response = wpt_musixmatch_get_lyrics( (string)$track->track_id );
        
        if( (string)$track_lyrics_xml_response->header->status_code != "200" ) {
          return send_module_errors("Musixmatch", "[API Failed] Failed to get lyrics for Track ID {$track->track_id}");
        }
        $track_title = substr((string)$track_lyrics_xml_response->body->lyrics->lyrics_body, 0, 25)."... By: ".(string)$track->artist_name;
        
        $content_search_tags = $wpt_musixmatch_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$track->track_name,
          (string)$track->artist_name,
          '<img src="'.(string)$track->album_coverart_100x100.'" alt="'.(string)$track_title.' Original" />',
          (string)$track->album_coverart_100x100,
          (string)$track_lyrics_xml_response->body->lyrics->lyrics_body,
          (string)$track_lyrics_xml_response->body->lyrics->lyrics_copyright,
          (string)$track_lyrics_xml_response->body->lyrics->script_tracking_url,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);


        $post_data = array(
          "uid" => md5((string)$track->track_id),
          "title" => (string)$track->track_name,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_musixmatch_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_musixmatch_module_template_installed") != "Yes") {
    $wpt->set_template("musixmatch", $wpt_musixmatch_default_module_template);
    update_option("wpt_musixmatch_module_template_installed", "Yes");
  }
  if(get_option("wpt_musixmatch_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_musixmatch_default_post_template);
    update_option("wpt_musixmatch_post_template_installed", "Yes");
  }
}
?>
