<?php

function wpt_recipes_options_admin($self) {
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_recipes_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  libxml_use_internal_errors(true);
  $default_params = array(
  );
  
  if( $options['wpt_recipes_exclude_imageless_recipes'] == "Yes" )
    $default_params['onlyImages'] = 1;
  
  $params = array_merge($params, $default_params);

  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  
  $request = "http://www.recipepuppy.com/?".$request_query;

  $response = wpt_get_response( $request, "Recipes" );
  $_response = phpQuery::newDocument($response);
  $output = array();
  $_recipe_rows = $_response->find("#right")->find("div.result");
  foreach( $_recipe_rows as $recipe_row ) {
    $ingredients = array();
    $_recipe_row = pq($recipe_row);
    $ings = $_recipe_row->find(".ings")->find("a.addIng");
    foreach( $ings as $ing ) {
      $ingredients[] = preg_replace("/^\+/", "", pq($ing)->text());
    }
    $output['recipe'][] = (object)array(
      "title" => trim($_recipe_row->find("h3:first")->text()),
      "url" => $_recipe_row->find("h3:first")->find("a:first")->attr("href"),
      "image_url" => $_recipe_row->find("img.thumb")->attr("src"),
      "summary" => $_recipe_row->find(".ings")->prev("div")->text(),
      "ingredients" => join(", ", $ingredients),
      "similar_recipes_url" => wpt_recipes_absolutize_url("http://www.recipepuppy.com", $_recipe_row->find(".url")->find("a")->attr("href")),
    );
  }
  return (object)$output;
}

function wpt_recipes_absolutize_url($base, $url) {
  if( strstr( $url, $base ) )
    return $url;
  else {
    $new_url = $base."/".preg_replace("/^\//", "", $url);
    return $new_url;
  }
}

function wpt_recipes_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_recipes_template_tags );
  }

  if($keywords == "") {
    return send_module_errors("Recipes", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'recipes'");
  if($template == false || empty($template)) {
    return send_module_errors("Recipes", "[No Template] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $recipes_xml_response = wpt_recipes_response(array(
                                        "q" => urlencode($keywords),
                                        "p" => $page,
                                      ));
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$recipes_xml_response) {
      return send_module_errors("Recipes", "[Request Failed] API request failed." );
    }

    if ( (count($recipes_xml_response->recipe) < 1) || !$recipes_xml_response->recipe) {
      return send_module_errors("Recipes", "[Content Empty] No recipes found for this keyword.");
    }
    foreach($recipes_xml_response->recipe as $recipe) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;

      if ($counter2<$count_this) {
        $content = $template;
        
        $content_search_tags = $wpt_recipes_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          $recipe->title,
          $recipe->url,
          $recipe->image_url,
          '<img src="'.$recipe->image_url.'" alt="'.$recipe->title.'" />',
          $recipe->summary,
          $recipe->ingredients,
          $recipe->similar_recipes_url,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5($recipe->url),
          "title" => $recipe->title,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );

        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_recipes_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_recipes_module_template_installed") != "Yes") {
    $wpt->set_template("recipes", $wpt_recipes_default_module_template);
    update_option("wpt_recipes_module_template_installed", "Yes");
  }
  if(get_option("wpt_recipes_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_recipes_default_post_template);
    update_option("wpt_recipes_post_template_installed", "Yes");
  }
}
?>
