<?php

function wpt_twitter_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_twitter_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  
  $default_params = array(
    "rpp"=>"10",
    "lang"=>"en",
    "result_type"=>$options['wpt_twitter_result_type']
  );

  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);
  $request = "http://search.twitter.com/search.json?".$request_query;

  $json_response = wpt_get_response( $request, "Twitter" );
  return json_decode( $json_response );
}

function wpt_twitter_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_twitter_template_tags );
  }

  if($keywords == "") {
    return send_module_errors("Twitter", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'twitter'");
  if($template == false || empty($template)) {
    return send_module_errors("Twitter", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();
  
  for($i = 0; $i < $count; $i++) {
    $twitter_json_response = wpt_twitter_response(array(
                                      "q" => urlencode($keywords),
                                      "page" => $page,
                                      ));
    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$twitter_json_response) {
      return send_module_errors("Twitter", "[Request Failed] API request failed." );
    }

    if(empty($twitter_json_response->results) || count( $twitter_json_response->results ) < 1)
      return send_module_errors("Twitter", "[Empty Content] No tweets were found for the keyword.");

    foreach($twitter_json_response->results as $tweet_item) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;
      if ($counter2<$count_this) {
        $content = $template;
        
        $content_search_tags = $wpt_twitter_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          $tweet_item->from_user_id_str,
          $tweet_item->profile_image_url,
          $tweet_item->created_at,
          $tweet_item->from_user,
          $tweet_item->text,
          "http://twitter.com/{$tweet_item->from_user}/statuses/{$tweet_item->id_str}",
          $tweet_item->id_str,
          $tweet_item->to_user,
          $tweet_item->geo,
          $tweet_item->to_user_id_str,
          $tweet_item->source,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5((string)$tweet_item->id_str),
          "title" => (string)$tweet_item->text,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_twitter_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_twitter_module_template_installed") != "Yes") {
    $wpt->set_template("twitter", $wpt_twitter_default_module_template);
    update_option("wpt_twitter_module_template_installed", "Yes");
  }
  if(get_option("wpt_twitter_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_twitter_default_post_template);
    update_option("wpt_twitter_post_template_installed", "Yes");
  }
}
?>
