<?php

function wpt_youtube_options_admin($self) {
  include(dirname(__file__)."/wptm-options.php");
  include_once(dirname(__file__)."/templates/admin-options.php");
}

function wpt_youtube_response($params) {
  $options = get_option("wpt_options");
  $options = is_array($options) ? $options : unserialize($options);
  $default_params = array(
    "safeSearch" => $options['wpt_youtube_safe_search'],
    "orderby" => $options['wpt_youtube_order_by'],
    "max-results" => "10",
    "v" => "2"
  );
  
  $params = array_merge($params, $default_params);
  ksort($params);

  $request_query = array();
  
  foreach ($params as $param=>$value) {
    $request_query[] = $param."=".$value;
  }
  $request_query = implode("&", $request_query);

  $request = "http://gdata.youtube.com/feeds/api/videos?".$request_query;

  return wpt_get_xml_response( $request, "Youtube" );
}

function wpt_youtube_post($params) {
  global $wpdb, $wpt;

  extract($params);
  include(dirname(__file__)."/wptm-options.php");
  $this_module_name = preg_replace("/^wptm\-/", "", basename(realpath(dirname(__FILE__))));
  
  if( isset($featured_image_module) && $featured_image_module == "{".$this_module_name."-fi}" ) {
    $featured_image_tt_index = array_search( "{".$featured_image_module_tag."}", $wpt_youtube_template_tags );
  }
  
  if($keywords == "") {
    return send_module_errors("Youtube", "[Keywords Empty] No keyword specified.");
  }
  
  $page = floor( $start/10 ) + 1;
  $last_post_number = $start - ( ($page - 1) * 10 );

  $template = $wpdb->get_var("SELECT content FROM {$wpt->db->table_name_templates} WHERE template_type = 'youtube'");
  if($template == false || empty($template)) {
    return send_module_errors("Youtube", "[Keywords Empty] Module Template was not found.");
  }

  $options = $wpt->options;

  $output = array();

  for($i = 0; $i < $count; $i++) {
    $response_array = array(
                            "q" => urlencode($keywords),
                            "start-index" => (!empty($start)) ? $start : 1,
                          );

    $youtube_xml_response = @wpt_youtube_response($response_array);

    $count_this = ($count <= 10) ? $count : 10;
    
    $counter2 = 0;
    $counter1 = 1;

    if (!$youtube_xml_response) {
      return send_module_errors("Youtube", "[Request Failed] API request failed." );
    }

    if (!isset($youtube_xml_response->entry) || empty( $youtube_xml_response->entry )) {
      return send_module_errors("Youtube", "[Request Failed] No results found for the keyword." );
    }


    foreach($youtube_xml_response->entry as $youtube_item) {
      if( $counter1 < $last_post_number ) {
        $counter1++;
        continue;
      }
      $counter1++;

      if ($counter2<$count_this) {
        $content = $template;
        
        $item_media = $youtube_item->children('http://search.yahoo.com/mrss/');
        $thumbnail_url = $item_media->group->thumbnail[0]->attributes();
        $thumbnail_url = $thumbnail_url['url'];
        $video_id = $item_media->children('http://gdata.youtube.com/schemas/2007');
        $embed_frame = '<iframe title="YouTube video player" class="youtube-player" type="text/html" width="640" height="390" src="http://www.youtube.com/embed/'.(string)$video_id->videoid.'" frameborder="0" allowFullScreen="true"></iframe>';
        
        $content_search_tags = $wpt_youtube_template_tags;
        
        $content_replace_tags = array(
          $keywords,
          (string)$item_media->group->title,
          (string)$thumbnail_url,
          (string)$video_id->videoid,
          (string)$item_media->group->description,
          $embed_frame,
          'http://www.youtube.com/embed/'.(string)$video_id->videoid,
        );
        
        $content = str_replace($content_search_tags, $content_replace_tags, $content);

        $post_data = array(
          "uid" => md5((string)$video_id->videoid),
          "title" => (string)$item_media->group->title,
          "content" => $content,
          "featured_image" => ( isset($featured_image_tt_index) && $featured_image_tt_index !== FALSE ) ? (string)$content_replace_tags[$featured_image_tt_index] : "",
        );
        array_push($output, $post_data);
        $counter2++;
      }
    }
    $page++;
    $last_post_number = 0;
  }
  return $output;
}

function wpt_youtube_init() {
  global $wpdb, $wpt;
  include(dirname(__file__)."/wptm-options.php");
  if(get_option("wpt_youtube_module_template_installed") != "Yes") {
    $wpt->set_template("youtube", $wpt_youtube_default_module_template);
    update_option("wpt_youtube_module_template_installed", "Yes");
  }
  if(get_option("wpt_youtube_post_template_installed") != "Yes") {
    $wpt->set_template("post", $wpt_youtube_default_post_template);
    update_option("wpt_youtube_post_template_installed", "Yes");
  }
}
?>
